<?php
//
//	Module name      :	odbc.php
//	Version          :	1.0
//	Started on       :	April 28, 2005
//	Completed on     :	June 27, 2007
//
//	Module type      :	Abstract type
//	Description      :	Functions to create accesses to databases using Open DataBase Connectivity layer
//
//	Developed under  :	Php 4.1.2
//	Sw requirements  :	None
//
//	Parameters       :	None
//	Notes - usage    :	This module can log performed queries using DbLog module and activating it
//						by $ODBC_db_log; queries are logged in a table named "ODBC" (see $ODBC_log_table_name
//						to change its name), with definition:
//							CREATE TABLE ODBC (
//								Ts int(11) NOT NULL default '0',
//								Ip varchar(15) NOT NULL default '',
//								Dsn varchar(32) NOT NULL default '',
//								Script varchar(24) NOT NULL default '',
//								Procedure varchar(8) NOT NULL default '',
//								Query text NOT NULL,
//								Result varchar(6) NOT NULL default '',
//								ErrorMsg varchar(127) NOT NULL default '',
//								Elapsed float(10,6) unsigned NOT NULL default '0.000000',
//								KEY Ts (Ts)
//							);
//						Other indexes can be added.
//						For an example of DB descriptors, see the global variable $ODBC_db_descriptor.
//


//--- Include files --------------------------------------------------------------------------------



require_once("dblog.php");



//--- Public constants -----------------------------------------------------------------------------



define("ODBC_MODULE_NAME", "odbc.php 1.0; on June 27, 2007 by J.Hubb.");

// ODBCSelect $mode values
define("ODBC_FIRST",	1);
define("ODBC_NEXT",		2);
define("ODBC_CLOSE",	3);

// Returned array type
define("ODBC_STD", 0);			// Standard array
define("ODBC_ASS", 1);			// Associative array
define("ODBC_FLAT", 2);			// Flat array (useful only for ODBCSelectManyRows, not an array of array)

// Error codes
define("ODBC_NO_ERROR",			0);
define("ODBC_HANDLE_ERROR",		1);
define("ODBC_RESOURCE_ERROR",	2);
define("ODBC_MODE_ERROR",		3);
define("ODBC_LAST_ERROR",		3);



//--- Public variables -----------------------------------------------------------------------------



$ODBC_db_log = FALSE;			// Change to TRUE to Log queries by DBLog module
$ODBC_log_table_name = "ODBC";	// Name of the table used by DBLog

$ODBC_db_descriptor	= array(	// Example for an uninitialized DB descriptor
		"Dsn"	=> "MyOdbcDsn",
		"User"	=> "MyUserName",
		"Pwd"	=> "MyPassword"
		);



//--- Private constants ----------------------------------------------------------------------------



//--- Private variables ----------------------------------------------------------------------------



$_ODBC_error_num = ODBC_NO_ERROR;
$_ODBC_error_msg = "No error";
$_ODBC_error_ext = array("Func" => "ODBC module");



//--- Public functions code ------------------------------------------------------------------------



function ODBCAffectedRows(&$desc)
// Returns the number of affected rows in the last query (it can be a select or any other query). It does
// not modify the error code, on errors returns 0.
{
	if (array_key_exists("_AffRows", $desc))
		return $desc["_AffRows"];
	else
		return 0;
}



function ODBCDone(&$desc)
// Closes the opened database connection.
{
	global $_ODBC_error_num, $_ODBC_error_msg, $_ODBC_error_ext, $ODBC_db_log, $ODBC_log_table_name;

	if (is_resource(@$desc["_Handle"]))
	{
		$t0 = microtime();
		@odbc_close(@$desc["_Handle"]);
		$t1 = microtime();
		unset($desc["_Handle"], $desc["_AffRows"]);
		$_ODBC_error_num = ODBC_NO_ERROR;
		$_ODBC_error_msg = "No error";
		$_ODBC_error_ext = array("Func" => "ODBCDone", "Dsn" => @$desc["Dsn"]);
	}
	else
	{
		$t0 = $t1 = "0 0";
		$_ODBC_error_num = ODBC_HANDLE_ERROR;
		$_ODBC_error_msg = "'" . @$desc["_Handle"] . "' is an invalid db handle (invalid ODBC resource)";
		$_ODBC_error_ext = array("Func" => "ODBCDone", "Dsn" => @$desc["Dsn"]);
	}
	if ($ODBC_db_log)
	{
		$tmp = pathinfo(@$_SERVER["SCRIPT_NAME"]);
		$t0b = explode(" ", $t0);
		$t1b = explode(" ", $t1);
		DLAppend($ODBC_log_table_name, @$_SERVER["REMOTE_ADDR"], @$desc["Dsn"], $tmp["basename"], "Init",
				"dummy", $_ODBC_error_num, $_ODBC_error_msg, number_format($t1b[1] - $t0b[1] + $t1b[0] -
				$t0b[0], 6));
	}
	return ($_ODBC_error_num === ODBC_NO_ERROR);
}



function ODBCErrorMsg()
// Returns the error message of the last module function called.
{
	global $_ODBC_error_msg;

	return $_ODBC_error_msg;
}



function ODBCExtErrorMsg()
// Returns the extended error messages (in Html format) of the last module function called.
{
	global $_ODBC_error_num, $_ODBC_error_msg, $_ODBC_error_ext;

	return "
		<table border=0 width=100%><tr><td colspan=2>" . ($_ODBC_error_num === ODBC_NO_ERROR ?
			"<b>$_ODBC_error_ext[Func]</b>" :
			"<font color=red><b>$_ODBC_error_ext[Func] ERROR</b></font>") .
		"</td></tr>" .
		(@$_ODBC_error_ext["Dsn"] == "" ?
			"" :
			"<tr><td width=1%><b>Dsn:</b></td><td>'$_ODBC_error_ext[Dsn]'</td></tr>") .
		(@$_ODBC_error_ext["Query"] == "" ?
			"" :
			"<tr><td width=1%><b>Query:</b></td><td>'$_ODBC_error_ext[Query]'</td></tr>") .
		"<tr><td width=1%><b>ErrorMsg:</b></td><td>'$_ODBC_error_msg'</td></tr>" .
		"</table>";
}



function ODBCInit(&$desc)
// Initializes a persistent connection with the database descripted in the associative array $desc; the array
// must have keys "Dsn", "User", "Pwd"; $desc is modified inserting the DB handle "_Handle". Returns TRUE if
// succeded, otherwise FALSE and error code and error message are set.
{
	global $_ODBC_error_num, $_ODBC_error_msg, $_ODBC_error_ext, $ODBC_db_log, $ODBC_log_table_name;

	$t0 = microtime();
	$desc["_Handle"] = @odbc_pconnect(@$desc["Dsn"], @$desc["User"], @$desc["Pwd"]);
	$t1 = microtime();
	if ($desc["_Handle"] === FALSE)
	{
		$_ODBC_error_num = odbc_error();
		$_ODBC_error_msg = odbc_errormsg();
		$_ODBC_error_ext = array("Func" => "ODBCInit", "Dsn" => @$desc["Dsn"]);
	}
	else
	{
		$_ODBC_error_num = ODBC_NO_ERROR;
		$_ODBC_error_msg = "No error";
		$_ODBC_error_ext = array("Func" => "ODBCInit", "Dsn" => @$desc["Dsn"]);
	}
	$desc["_AffRows"] = 0;
	if ($ODBC_db_log)
	{
		$tmp = pathinfo(@$_SERVER["SCRIPT_NAME"]);
		$t0b = explode(" ", $t0);
		$t1b = explode(" ", $t1);
		DLAppend($ODBC_log_table_name, @$_SERVER["REMOTE_ADDR"], @$desc["Dsn"], $tmp["basename"], "Init",
				"dummy", $_ODBC_error_num, $_ODBC_error_msg, number_format($t1b[1] - $t0b[1] + $t1b[0] -
				$t0b[0], 6));
	}
	return ($_ODBC_error_num === ODBC_NO_ERROR);
}



function ODBCQuery(&$desc, $query)
// Performs the generic query $query (without returning values), on the database descripted in the
// associative array $desc. If the query fails, return FALSE and error code and error message are set.
{
	global $_ODBC_error_num, $_ODBC_error_msg, $_ODBC_error_ext, $ODBC_db_log, $ODBC_log_table_name;

	$t0 = microtime();
	$desc["_Result"] = @odbc_exec(@$desc["_Handle"], $query);
	$t1 = microtime();
	if ($desc["_Result"] == FALSE)
	{
		$desc["_AffRows"] = 0;
		if ($desc["_Result"] === FALSE)
		{
			$_ODBC_error_num = odbc_error();
			$_ODBC_error_msg = odbc_errormsg();
			$_ODBC_error_ext = array("Func" => "ODBCQuery", "Dsn" => @$desc["Dsn"], "Query" => $query);
		}
		else
		{
			$_ODBC_error_num = ODBC_HANDLE_ERROR;
			$_ODBC_error_msg = "'" . @$desc["_Handle"] . "' is an invalid db handle (invalid ODBC resource)";
			$_ODBC_error_ext = array("Func" => "ODBCQuery", "Dsn" => @$desc["Dsn"], "Query" => $query);
		}
	}
	else
	{
		$desc["_AffRows"] = @odbc_num_rows($desc["_Result"]);
		$_ODBC_error_num = ODBC_NO_ERROR;
		$_ODBC_error_msg = "No error";
		$_ODBC_error_ext = array("Func" => "ODBCQuery", "Dsn" => @$desc["Dsn"], "Query" => $query);
	}
	if ($ODBC_db_log)
	{
		$tmp = pathinfo(@$_SERVER["SCRIPT_NAME"]);
		$t0b = explode(" ", $t0);
		$t1b = explode(" ", $t1);
		DLAppend($ODBC_log_table_name, @$_SERVER["REMOTE_ADDR"], @$desc["Dsn"], $tmp["basename"], "Query",
				$query, $_ODBC_error_num, $_ODBC_error_msg, number_format($t1b[1] - $t0b[1] + $t1b[0] -
				$t0b[0], 6));
	}
	return ($_ODBC_error_num === ODBC_NO_ERROR);
}



function ODBCResult()
// Returns the error code of the last module function called.
{
	global $_ODBC_error_num;

	return $_ODBC_error_num;
}



function ODBCSelect(&$desc, $query, $mode, $array_type = ODBC_STD)
// Performs the query $query on the database descripted in the associative array $desc, according to $mode,
// returning a tuple for each call in an array; this can have numeric indexes (default) or the names of the
// requested fields ($array_type = ODBC_ASS; ODBC_FLAT is treated as ODBC_STD). $desc is modified inserting
// the result handle "_Result".
// The first call must be with $mode set to ODBC_FIRST, next ones with $mode set to ODBC_NEXT ($query is
// ignored); after the last result the select must be closed with $mode set to $ODBC_CLOSE.  If the query
// fails, return FALSE and error code and error message are set.
{
	global $_ODBC_error_num, $_ODBC_error_msg, $_ODBC_error_ext, $ODBC_db_log, $ODBC_log_table_name;

    switch ($mode)
	{
		case ODBC_FIRST:
			$t0 = microtime();
			$desc["_Result"] = @odbc_exec($desc["_Handle"], $query);
			$t1 = microtime();
			if ($desc["_Result"] == FALSE)
			{
				$desc["_AffRows"] = 0;
				if ($desc["_Result"] === FALSE)
				{
					$_ODBC_error_num = odbc_error();
					$_ODBC_error_msg = odbc_errormsg();
					$_ODBC_error_ext = array("Func" => "ODBCSelect(First)", "Dsn" => @$desc["Dsn"],
							"Query" => $query);
				}
				else
				{
					$_ODBC_error_num = ODBC_HANDLE_ERROR;
					$_ODBC_error_msg = "'" . @$desc["_Handle"] .
							"' is an invalid db handle (invalid ODBC resource)";
					$_ODBC_error_ext = array("Func" => "ODBCSelect(First)", "Dsn" => @$desc["Dsn"],
							"Query" => $query);
				}
				$result = FALSE;
			}
			else
			{
				@odbc_fetch_into($desc["_Result"], $result);
				if ($array_type === ODBC_ASS)
				{
					$desc["_ResultFields"] = _ODBCGetFieldNames($desc["_Result"]);
					$result_ass = array();
					foreach ($result as $n => $val)
						$result_ass[$desc["_ResultFields"][$n]] = $val;
					$result = $result_ass;
				}
				$desc["_AffRows"] = (count($result) == 0 ? 0 : 1);
				$_ODBC_error_num = ODBC_NO_ERROR;
				$_ODBC_error_msg = "No error";
				$_ODBC_error_ext = array("Func" => "ODBCSelect(First)", "Dsn" => @$desc["Dsn"],
						"Query" => $query);
			}
			if ($ODBC_db_log)
			{
				$tmp = pathinfo(@$_SERVER["SCRIPT_NAME"]);
				$t0b = explode(" ", $t0);
				$t1b = explode(" ", $t1);
				DLAppend($ODBC_log_table_name, @$_SERVER["REMOTE_ADDR"], @$desc["Dsn"], $tmp["basename"],
						"Select", $query, $_ODBC_error_num, $_ODBC_error_msg, number_format($t1b[1] -
						$t0b[1] + $t1b[0] - $t0b[0], 6));
			}
			return $result;
		case ODBC_NEXT:
			$tmp = @odbc_fetch_into(@$desc["_Result"], $result);
			if (($tmp == FALSE) && ($tmp !== FALSE))
			{
				$desc["_AffRows"] = 0;
				$_ODBC_error_num = ODBC_RESOURCE_ERROR;
				$_ODBC_error_msg = "'" . @$desc["_Result"] .
						"' is an invalid resource handle (invalid ODBC resource)";
				$_ODBC_error_ext = array("Func" => "ODBCSelect(Next)", "Dsn" => @$desc["Dsn"]);
				$result = FALSE;
			}
			else
			{
				if ($array_type === ODBC_ASS)
				{
					$result_ass = array();
					foreach ($result as $n => $val)
						$result_ass[@$desc["_ResultFields"][$n]] = $val;
					$result = $result_ass;
				}
				$desc["_AffRows"] = (count($result) == 0 ? 0 : 1);
				$_ODBC_error_num = ODBC_NO_ERROR;
				$_ODBC_error_msg = "No error";
				$_ODBC_error_ext = array("Func" => "ODBCSelect(Next)", "Dsn" => @$desc["Dsn"]);
			}
			return $result;
		case ODBC_CLOSE:
			if (@odbc_free_result(@$desc["_Result"]) == FALSE)
			{
				$_ODBC_error_num = ODBC_RESOURCE_ERROR;
				$_ODBC_error_msg = "'" . @$desc["_Result"] .
						"' is an invalid resource handle (invalid ODBC resource)";
				$_ODBC_error_ext = array("Func" => "ODBCSelect(Close)", "Dsn" => @$desc["Dsn"]);
				$result = FALSE;
			}
			else
			{
				unset($desc["_Result"]);
				unset($desc["_ResultFields"]);
				$_ODBC_error_num = ODBC_NO_ERROR;
				$_ODBC_error_msg = "No error";
				$_ODBC_error_ext = array("Func" => "ODBCSelect(Close)", "Dsn" => @$desc["Dsn"]);
				$result = TRUE;
			}
			return $result;
		default:
			$desc["_AffRows"] = 0;
			$_ODBC_error_num = ODBC_MODE_ERROR;
			$_ODBC_error_msg = "'$mode' is not a valid ODBCSelect mode";
			$_ODBC_error_ext = array("Func" => "ODBCSelect($mode)", "Dsn" => @$desc["Dsn"], "Query" => $query);
			return FALSE;
	}
}



function ODBCSelectManyRows(&$desc, $query, $array_type = ODBC_STD)
// Performs the query $query on the database descripted in the associative array $desc, returning the
// all the resulting tuples in an array of arrays for $array_type = ODBC_STD (indexes are numeric), in an
// array of associative arrays for $array_type = ODBC_ASS (internal indexes are the names of the requested
// fields), in a simple flat array for $array_type = ODBC_FLAT (with n fields in the query, first tuple have
// numeric indexes 0..n, second tuple (n + 1)..(2 * n), etc.). Use with caution, since this function could
// be very time and memory consuming! If the query fails, return FALSE and error code and error message are
// set.
{
	global $_ODBC_error_num, $_ODBC_error_msg, $_ODBC_error_ext, $ODBC_db_log, $ODBC_log_table_name;

	$result = array();
	$t0 = microtime();
	$desc["_Result"] = @odbc_exec($desc["_Handle"], $query);
	$t1 = microtime();
	if ($desc["_Result"] == FALSE)
	{
		$desc["_AffRows"] = 0;
		if ($desc["_Result"] === FALSE)
		{
			$_ODBC_error_num = odbc_error();
			$_ODBC_error_msg = odbc_errormsg();
			$_ODBC_error_ext = array("Func" => "ODBCSelectManyRows", "Dsn" => @$desc["Dsn"],
					"Query" => $query);
		}
		else
		{
			$_ODBC_error_num = ODBC_HANDLE_ERROR;
			$_ODBC_error_msg = "'" . @$desc["_Handle"] . "' is an invalid db handle (invalid ODBC resource)";
			$_ODBC_error_ext = array("Func" => "ODBCSelectManyRows", "Dsn" => @$desc["Dsn"],
					"Query" => $query);
		}
		$result = FALSE;
	}
	else
	{
		switch ($array_type)
		{
			case ODBC_ASS:
				$field_names = _ODBCGetFieldNames($desc["_Result"]);
				$row_ass = array();
				break;
			case ODBC_FLAT:
				$field_num = odbc_num_fields($desc["_Result"]);
				break;
		}
		while (@odbc_fetch_into($desc["_Result"], $row))
		{
			switch ($array_type)
			{
				case ODBC_ASS:
					foreach ($row as $n => $val)
						$row_ass[$field_names[$n]] = $val;
					array_push($result, $row_ass);
					break;
				case ODBC_FLAT:
					$result = array_merge($result, $row);
					break;
				default:		// Default to ODBC_STD.
					array_push($result, $row);
					break;
			}
		}
		@odbc_free_result($desc["_Result"]);
		unset($desc["_Result"]);
		if ($array_type === ODBC_FLAT)
			$desc["_AffRows"] = count($result) / $field_num;
		else
			$desc["_AffRows"] = count($result);
		$_ODBC_error_num = ODBC_NO_ERROR;
		$_ODBC_error_msg = $txt_msg = "No error";
		$_ODBC_error_ext = array("Func" => "ODBCSelectManyRows", "Dsn" => @$desc["Dsn"], "Query" => $query);
	}
	if ($ODBC_db_log)
	{
		$tmp = pathinfo(@$_SERVER["SCRIPT_NAME"]);
		$t0b = explode(" ", $t0);
		$t1b = explode(" ", $t1);
		DLAppend($ODBC_log_table_name, @$_SERVER["REMOTE_ADDR"], @$desc["Dsn"], $tmp["basename"], "SelManyR",
				$query, $_ODBC_error_num, $_ODBC_error_msg, number_format($t1b[1] - $t0b[1] + $t1b[0] -
				$t0b[0], 6));
	}
	return $result;
}



function ODBCSelectOneRow(&$desc, $query, $array_type = ODBC_STD)
// Performs the query $query on the database descripted in the associative array $desc, returning only
// the first resulting tuple in an array; this can have numeric indexes (default) or the names of the
// requested fields ($array_type = ODBC_ASS; ODBC_FLAT is treated as ODBC_STD). If the query fails,
// return FALSE and error code and error message are set.
{
	global $ODBC_db_log, $_ODBC_error_num, $_ODBC_error_msg, $_ODBC_error_ext, $ODBC_log_table_name;

	$tmp_log = $ODBC_db_log;
	$ODBC_db_log = FALSE;
	$t0 = microtime();
	$row = ODBCSelect($desc, $query, ODBC_FIRST, $array_type);
	$t1 = microtime();
	$tmp_err = $_ODBC_error_num;
	$tmp_msg = $_ODBC_error_msg;
	ODBCSelect($desc, "", ODBC_CLOSE);
	$ODBC_db_log = $tmp_log;
	$_ODBC_error_num = $tmp_err;
	$_ODBC_error_msg = $tmp_msg;
	$_ODBC_error_ext = array("Func" => "ODBCSelectOneRow", "Dsn" => @$desc["Dsn"], "Query" => $query);
	if ($ODBC_db_log)
	{
		$tmp = pathinfo(@$_SERVER["SCRIPT_NAME"]);
		$t0b = explode(" ", $t0);
		$t1b = explode(" ", $t1);
		DLAppend($ODBC_log_table_name, @$_SERVER["REMOTE_ADDR"], @$desc["Dsn"], $tmp["basename"], "SelOneR",
				$query, $_ODBC_error_num, $_ODBC_error_msg, number_format($t1b[1] - $t0b[1] + $t1b[0] -
				$t0b[0], 6));
	}
	return $row;
}



//--- Private functions code -----------------------------------------------------------------------



function _ODBCGetFieldNames($res_id)
// Returns an array (numbered from 0) with all field names as values.
{
	$result = array();
	for ($n = 1; $n <= odbc_num_fields($res_id); $n++)
			$result[] = odbc_field_name($res_id, $n);
	return $result;
}



//--- Initialization code --------------------------------------------------------------------------



?>
